/*
 * Dinorms: natural aggregation of dichotomic values.
 * (C) Martin Saturka, http://enduce.tangloid.net/
 * Version 0.92, released under the MIT license.
**/

#ifndef __DINORMS_H__
#define __DINORMS_H__

namespace Enduce {

const int AGGTYPESCNT = 3;
const int CMBTYPESCNT = 4;

enum aggtypes {AGGMAXIMUM, AGGCOPRODUCT, AGGSUMMATION};
enum cmbtypes {CMBMAXIMUM, CMBCODIVISION, CMBSUBTRACTION, CMBSTABILIZED};

class Dinorms {

  public:
    Dinorms(void);
    Dinorms(double thragg, double thrcmb);

    bool thresholds(double thragg, double thrcmb);
    int thresholds(int thlen, double *thres);
    bool setModes(int aggtype, int cmbtype);
    int getModes(int mdlen, int *modes);

    bool aggregate(int vallen, double *values, double *results);

  private:
    bool aggType0(int vallen, double *values, double *results);
    bool aggType1(int vallen, double *values, double *results);
    bool aggType2(int vallen, double *values, double *results);

    bool cmbType0(double *combs);
    bool cmbType1(double *combs);
    bool cmbType2(double *combs);
    bool cmbType3(double *combs);

    double m_thragg;
    double m_thrcmb;

    int m_aggtype;
    int m_cmbtype;

};

extern Dinorms *din;

}

#endif //__DINORMS_H__


