#Clustions functions of the Enduce library

clustions.thresholds <- function(thresh=NULL)
{
  if (!is.null(thresh)) {
    if (!is.vector(thresh))
      stop("threshold should be a float-point number")
    if (1 != length(thresh))
      stop("threshold should be a float-point number")
    if (!is.double(thresh))
      stop("threshold should be a float-point number")
    if (0 > thresh[1])
      stop("threshold should be >= 0")
    if (1 < thresh[1])
      stop("threshold should be <= 1")

    .C("clu_setThresholds",
      as.integer(1),
      as.double(thresh),
      integer(1))
  }

  res <- .C("clu_getThresholds",
    as.integer(1),
    thresh = double(1),
    integer(1))

  return(list(thr=res$thresh))
}

clustions.maxcycles <- function(maxcyc=NULL)
{
  if (!is.null(maxcyc[1])) {
    if (!is.vector(maxcyc))
      stop("maxcyc should be an integer")
    if (1 != length(maxcyc))
      stop("maxcyc should be an integer")
    if (is.double(maxcyc))
      maxcyc = as.integer(maxcyc)
    if (!is.integer(maxcyc))
      stop("maxcyc should be an integer")
    if (0 >= maxcyc[1])
      stop("maxcyc should be > 0")

    .C("clu_setLimits",
      as.integer(1),
      as.integer(maxcyc),
      integer(1))
  }

  res <- .C("clu_getLimits",
    as.integer(1),
    maxcyc = integer(1),
    integer(1))

  return(list(maxcycles=res$maxcyc))
}

clustions.cluster <- function(data.cases=NULL, ini.centers=NULL, by.rows=FALSE)
{
  if (!is.matrix(data.cases))
    stop("data.cases should be a matrix")
  if (!is.double(data.cases))
    stop("data.cases should be a matrix of float-point numbers")
  if (0 < length(data.cases[is.na(data.cases)]))
    stop("NAs are not allowed here")
  if (0 > min(data.cases))
    stop("data.cases values should be in [0, 1] interval")
  if (1 < max(data.cases))
    stop("data.cases values should be in [0, 1] interval")

  if (!is.matrix(ini.centers))
    stop("ini.centers should be a matrix")
  if (!is.double(ini.centers))
    stop("ini.centers should be a matrix of float-point numbers")
  if (0 < length(ini.centers[is.na(ini.centers)]))
    stop("NAs are not allowed here")
  if (0 > min(ini.centers))
    stop("ini.centers values should be in [0, 1] interval")
  if (1 < max(ini.centers))
    stop("ini.centers values should be in [0, 1] interval")

  data.dims = attributes(data.cases)$dim
  cens.dims = attributes(ini.centers)$dim

  if (!is.vector(by.rows))
    stop("by.rows should be a logical value")
  if (1 != length(by.rows))
    stop("by.rows should be a logical value")
  if (!is.logical(by.rows))
    stop("by.rows should be a logical value")

  if (by.rows[1]) {
    if (data.dims[2] != cens.dims[2])
      stop("matrices should have the same column count")
    data.cases <- t(data.cases)
    ini.centers <- t(ini.centers)
    data.dims = attributes(data.cases)$dim
    cens.dims = attributes(ini.centers)$dim
  } else {
    if (data.dims[1] != cens.dims[1])
      stop("matrices should have the same row count")
  }

  if (0 == cens.dims[1])
    stop("matrices should have non-zero sizes")
  if (0 == data.dims[1])
    stop("matrices should have non-zero sizes")
  if (0 == data.dims[2])
    stop("matrices should have non-zero sizes")

  sizes <- c(cens.dims[2], data.dims[2], data.dims[1])
  case.count <- data.dims[2]

  res <- .C("clu_cluster",
    centers = as.double(ini.centers),
    as.double(data.cases),
    as.integer(sizes),
    grains = integer(case.count),
    simils = double(case.count),
    info = integer(2),
    state = integer(1))

  if (0 == res$state[1]) {
    return(list(state=FALSE))
  }

  res.grains <- apply(as.matrix(c(1:case.count)), 1, function(ind) {res$grains[ind] + 1})

  res.centers <- matrix(res$centers, nrow=cens.dims[1], ncol=cens.dims[2], byrow=FALSE)
  if (by.rows[1]) {
    res.centers <- t(res.centers)
  }

  return(list(state=TRUE, converged=res$info[1], cycles=res$info[1],
    centers=res.centers, grains=res.grains, similarities=res$simils))
}

clustions.cluster2 <- function(data.cases=NULL, ini.centers=NULL, by.rows=FALSE)
{
  if (!is.matrix(data.cases))
    stop("data.cases should be a matrix")
  if (!is.double(data.cases))
    stop("data.cases should be a matrix of float-point numbers")
  if (0 < length(data.cases[is.na(data.cases)]))
    stop("NAs are not allowed here")
  if (-1 > min(data.cases))
    stop("data.cases values should be in [-1, 1] interval")
  if (1 < max(data.cases))
    stop("data.cases values should be in [-1, 1] interval")

  if (!is.matrix(ini.centers))
    stop("ini.centers should be a matrix")
  if (!is.double(ini.centers))
    stop("ini.centers should be a matrix of float-point numbers")
  if (0 < length(ini.centers[is.na(ini.centers)]))
    stop("NAs are not allowed here")
  if (0 > min(ini.centers))
    stop("ini.centers values should be in [0, 1] interval")
  if (1 < max(ini.centers))
    stop("ini.centers values should be in [0, 1] interval")

  data.dims = attributes(data.cases)$dim
  cens.dims = attributes(ini.centers)$dim

  if (!is.vector(by.rows))
    stop("by.rows should be a vector")
  if (!is.logical(by.rows))
    stop("by.rows should be a logical vector")

  if (by.rows[1]) {
    if (data.dims[2] != cens.dims[2])
      stop("matrices should have the same column count")
    data.cases <- t(data.cases)
    ini.centers <- t(ini.centers)
    data.dims = attributes(data.cases)$dim
    cens.dims = attributes(ini.centers)$dim
  } else {
    if (data.dims[1] != cens.dims[1])
      stop("matrices should have the same row count")
  }

  if (0 == cens.dims[1])
    stop("matrices should have non-zero sizes")
  if (0 == data.dims[1])
    stop("matrices should have non-zero sizes")
  if (0 == data.dims[2])
    stop("matrices should have non-zero sizes")

  sizes <- c(cens.dims[2], data.dims[2], data.dims[1])
  case.count <- data.dims[2]

  res <- .C("clu_cluster2",
    centers = as.double(ini.centers),
    as.double(data.cases),
    as.integer(sizes),
    grainsneg = integer(case.count),
    grainspos = integer(case.count),
    similsneg = double(case.count),
    similspos = double(case.count),
    info = integer(2),
    state = integer(1))

  if (0 == res$state[1]) {
    return(list(state=FALSE))
  }

  res.grainsneg <- apply(as.matrix(c(1:case.count)), 1, function(ind) {res$grainsneg[ind] + 1})
  res.grainspos <- apply(as.matrix(c(1:case.count)), 1, function(ind) {res$grainspos[ind] + 1})

  res.centers <- matrix(res$centers, nrow=cens.dims[1], ncol=cens.dims[2], byrow=FALSE)
  if (by.rows[1]) {
    res.centers <- t(res.centers)
  }

  return(list(state=TRUE, converged=res$info[1], cycles=res$info[1],
    centers=res.centers, grains.neg=res.grainsneg, grains.pos=res.grainspos,
    similarities.neg=res$similsneg, similarities.pos=res$similspos))
}



