#Contifiers functions of the Enduce library

contifiers.thresholds <- function(thresh=NULL)
{
  if (!is.null(thresh)) {
    if (!is.vector(thresh))
      stop("threshold should be a float-point number")
    if (1 != length(thresh))
      stop("threshold should be a float-point number")
    if (!is.double(thresh))
      stop("threshold should be a float-point number")
    if (0 > thresh[1])
      stop("threshold should be >= 0")
    if (1 < thresh[1])
      stop("threshold should be <= 1")

    .C("con_setThresholds",
      as.integer(1),
      as.double(thresh),
      integer(1))
  }

  res <- .C("con_getThresholds",
    as.integer(1),
    thresh = double(1),
    integer(1))

  return(list(thr=res$thresh))
}

contifiers.modes <- function(modes=NULL)
{
  DIRTYPES <- c("mutual", "directional", "inverse")
  AGGTYPES <- c("minimum", "product", "bold")

  if (!is.null(modes)) {
    if (!is.vector(modes))
      stop("modes should be c(dir_type, agg_type)")
    if (2 != length(modes))
      stop("modes should be c(dir_type, agg_type)")
    if (!is.character(modes))
      stop("modes should be c(dir_type, agg_type)")

    typdir <- modes[1]
    typagg <- modes[2]

    if (!is.character(typdir))
      stop("directionality type should be mutual/directional/inverse")
    t1 <- pmatch(typdir, DIRTYPES)
    if(is.na(t1))
      stop("directionality type should be mutual/directional/inverse")
    t1 <- t1 - 1

    if (!is.character(typagg))
      stop("aggregation type should be minimum/product/bold")
    t2 <- pmatch(typagg, AGGTYPES)
    if(is.na(t2))
      stop("aggregation type should be minimum/product/bold")
    t2 <- t2 - 1

    imodes <- c(t1, t2)

    .C("con_setModes",
      as.integer(2),
      as.integer(imodes),
      integer(1))
  }

  res <- .C("con_getModes",
    as.integer(2),
    imodes = integer(2),
    integer(1))

  typdir <- DIRTYPES[(res$imodes[1]) + 1]
  typagg <- AGGTYPES[(res$imodes[2]) + 1]

  return(list(dir=typdir, agg=typagg))
}

contifiers.aggregate <- function(vecx=NULL, vecy=NULL)
{
  if (!is.vector(vecx))
    stop("the first argument should be a vector")
  if (!is.double(vecx))
    stop("the first argument should be a vector of float-point numbers")
  if (0 < length(vecx[is.na(vecx)]))
    stop("NAs are not allowed here")
  if (0 > min(vecx))
    stop("vector values should be in [0, 1] interval")
  if (1 < max(vecx))
    stop("vector values should be in [0, 1] interval")

  if (!is.vector(vecy))
    stop("the second argument should be a vector")
  if (!is.double(vecy))
    stop("the second argument should be a vector of float-point numbers")
  if (0 < length(vecy[is.na(vecy)]))
    stop("NAs are not allowed here")
  if (0 > min(vecy))
    stop("vector values should be in [0, 1] interval")
  if (1 < max(vecy))
    stop("vector values should be in [0, 1] interval")

  if (length(vecx) != length(vecy))
    stop("vector lengths should be same")

  res <- .C("con_aggregate",
    as.integer(length(vecx)),
    as.double(vecx),
    as.double(vecy),
    results = double(1),
    state = integer(1))

  if (0 == res$state[1]) {
    return(list(state=FALSE))
  }

  return(list(state=TRUE, results=res$results))
}



