#Dinorms functions of the Enduce library

dinorms.thresholds <- function(thresh=NULL)
{
  if (!is.null(thresh)) {
    if (!is.vector(thresh))
      stop("thresholds should be two float-point numbers")
    if (2 != length(thresh))
      stop("thresholds should be two float-point numbers")
    if (!is.double(thresh))
      stop("thresholds should be two float-point numbers")
    if (0 > thresh[1])
      stop("thresholds should be >= 0")
    if (1 < thresh[1])
      stop("thresholds should be <= 1")
    if (0 > thresh[2])
      stop("thresholds should be >= 0")
    if (1 < thresh[2])
      stop("thresholds should be <= 1")

    .C("din_setThresholds",
      as.integer(2),
      as.double(thresh),
      integer(1))
  }

  res <- .C("din_getThresholds",
    as.integer(2),
    thresh = double(2),
    integer(1))

  return(list(thr=res$thresh))
}

dinorms.modes <- function(modes=NULL)
{
  ADDTYPES <- c("maximum", "coproduct", "summation")
  CMBTYPES <- c("maximum", "codivision", "subtraction", "stabilized")

  if (!is.null(modes)) {
    if (!is.vector(modes))
      stop("modes should be c(add_type, cmb_type)")
    if (2 != length(modes))
      stop("modes should be c(add_type, cmb_type)")
    if (!is.character(modes))
      stop("modes should be c(add_type, cmb_type)")

    typadd <- modes[1]
    typcmb <- modes[2]

    if (!is.character(typadd))
      stop("addition type should be maximum/coproduct/summation")
    t1 <- pmatch(typadd, ADDTYPES)
    if(is.na(t1))
      stop("addition type should be maximum/coproduct/summation")
    t1 <- t1 - 1

    if (!is.character(typcmb))
      stop("combination type should be maximum/codivision/subtraction/stabilized")
    t2 <- pmatch(typcmb, CMBTYPES)
    if(is.na(t2))
      stop("combination type should be maximum/codivision/subtraction/stabilized")
    t2 <- t2 - 1

    imodes <- c(t1, t2)

    .C("din_setModes",
      as.integer(2),
      as.integer(imodes),
      integer(1))
  }

  res <- .C("din_getModes",
    as.integer(2),
    imodes = integer(2),
    integer(1))

  typadd <- ADDTYPES[(res$imodes[1]) + 1]
  typcmb <- CMBTYPES[(res$imodes[2]) + 1]

  return(list(add=typadd, cmb=typcmb))
}

dinorms.aggregate <- function(vect=NULL)
{
  if (!is.vector(vect))
    stop("vect should be a vector")
  if (!is.double(vect))
    stop("vect should be a vector of float-point numbers")
  if (0 < length(vect[is.na(vect)]))
    stop("NAs are not allowed here")
  if (-1 > min(vect))
    stop("vector values should be in [-1, 1] interval")
  if (1 < max(vect))
    stop("vector values should be in [-1, 1] interval")

  res <- .C("din_aggregate",
    as.integer(length(vect)),
    as.double(vect),
    results = double(3),
    state = integer(1))

  if (0 == res$state[1]) {
    return(list(state=FALSE))
  }

  return(list(state=TRUE, results=res$results))
}



